@@ -9,144 +9,83 @@ from uiautomator import AutomatorServer
 
 class TestAutomatorServer(unittest.TestCase):
 
-    def test_device_serial(self):
-        serial = "76HDGKDN783HD6D"
-        with patch("uiautomator.adb") as adb:
-            adb.devices = {serial: "device"}
-            with patch.dict('os.environ', {}, clear=True):
-                self.assertEqual(AutomatorServer().device_serial(), serial)
-                self.assertTrue("ANDROID_SERIAL" in os.environ)
-                self.assertEqual(os.environ["ANDROID_SERIAL"], serial)
-            with patch.dict('os.environ', {"ANDROID_SERIAL": "XXX"}):
-                self.assertEqual(AutomatorServer().device_serial(), "XXX")
-            adb.devices = {}
-            with self.assertRaises(EnvironmentError):
-                AutomatorServer().device_serial()
-            adb.devices = {serial: "device", "AAA": "device2"}
-            with patch.dict('os.environ', {}, clear=True):
-                with self.assertRaises(EnvironmentError):
-                    AutomatorServer().device_serial()
-            with patch.dict('os.environ', {"ANDROID_SERIAL": "XXX"}):
-                with self.assertRaises(EnvironmentError):
-                    AutomatorServer().device_serial()
-            with patch.dict('os.environ', {"ANDROID_SERIAL": serial}):
-                self.assertEqual(AutomatorServer().device_serial(), serial)
-                self.assertTrue("ANDROID_SERIAL" in os.environ)
-                self.assertEqual(os.environ["ANDROID_SERIAL"], serial)
-
     def test_local_port(self):
-        serial = "76HDGKDN783HD6D"
-        with patch("uiautomator.adb") as adb:
-            server = AutomatorServer()
-            server.device_serial = MagicMock()
-            server.device_serial.return_value = serial
-            adb.forward_list = {serial: [9009, 9001], "xxx": [123, 9001]}
-            self.assertEqual(server.local_port, 9009)
-            adb.forward_list = {"aaa": [9009, 9001], "xxx": [123, 9001]}
-            # the second time will retrieve the stored one.
-            self.assertEqual(server.local_port, 9009)
+        for port in range(9000, 9100):
+            with patch.dict('os.environ', {'LOCAL_PORT': str(port)}):
+                self.assertEqual(AutomatorServer().local_port, port)
+        with patch.dict('os.environ', {}, clear=True):
+            self.assertEqual(AutomatorServer().local_port, 9008)
 
-            server = AutomatorServer()
-            server.device_serial = MagicMock()
-            server.device_serial.return_value = serial
-            adb.forward_list = {"aaa": [9009, 9001], "xxx": [123, 9001]}
-            self.assertEqual(server.local_port, None)
+    def test_device_port(self):
+        for port in range(9000, 9100):
+            with patch.dict('os.environ', {'DEVICE_PORT': str(port)}):
+                self.assertEqual(AutomatorServer().device_port, port)
+        with patch.dict('os.environ', {}, clear=True):
+            self.assertEqual(AutomatorServer().device_port, 9008)
 
     def test_start_success(self):
-        serial = "76HDGKDN783HD6D"
         server = AutomatorServer()
         server.download_and_push = MagicMock()
         server.download_and_push.return_value = ["bundle.jar", "uiautomator-stub.jar"]
-        server.device_serial = MagicMock()
-        server.device_serial.return_value = serial
         server.ping = MagicMock()
         server.ping.return_value = "pong"
         with patch("uiautomator.adb") as adb:
-            with patch("uiautomator.server_port") as server_port:
-                adb.forward_list = {"a": [9008, 9008], "b": [9009, 9008]}
-                server_port.return_value = 9000
-                adb.forward.return_value = 0
+            adb.forward.return_value = 0
+            with patch.dict('os.environ', {'LOCAL_PORT': '9000', 'DEVICE_PORT': '9000'}):
                 server.start()
-                adb.forward.assert_called_once_with(9010, 9000)
+                adb.cmd.assert_valled_onec_with('shell', 'uiautomator', 'runtest', 'bundle.jar', 'uiautomator-stub.jar', '-c', 'com.github.uiautomatorstub.Stub')
+                adb.forward.assert_called_once_with(9000, 9000)
 
     def test_start_forward_error(self):
-        serial = "76HDGKDN783HD6D"
         server = AutomatorServer()
         server.download_and_push = MagicMock()
         server.download_and_push.return_value = ["bundle.jar", "uiautomator-stub.jar"]
-        server.device_serial = MagicMock()
-        server.device_serial.return_value = serial
         with patch("uiautomator.adb") as adb:
-            with patch("uiautomator.server_port") as server_port:
-                adb.forward_list = {"a": [9008, 9008], "b": [9009, 9008]}
-                server_port.return_value = 9000
-                adb.forward.return_value = 1
+            adb.forward.return_value = 1
+            with patch.dict('os.environ', {'LOCAL_PORT': '9000', 'DEVICE_PORT': '9000'}):
                 with self.assertRaises(IOError):
                     server.start()
-                assert adb.forward.call_count == 190
+                adb.forward.assert_called_once_with(9000, 9000)
 
     def test_start_error(self):
-        serial = "76HDGKDN783HD6D"
         server = AutomatorServer()
         server.download_and_push = MagicMock()
         server.download_and_push.return_value = ["bundle.jar", "uiautomator-stub.jar"]
-        server.device_serial = MagicMock()
-        server.device_serial.return_value = serial
         server.ping = MagicMock()
         server.ping.return_value = None
         with patch("uiautomator.adb") as adb:
-            with patch("uiautomator.server_port") as server_port:
-                adb.forward_list = {"a": [9008, 9008], "b": [9009, 9008]}
-                server_port.return_value = 9000
-                adb.forward.return_value = 0
+            adb.forward.return_value = 0
+            with patch.dict('os.environ', {'LOCAL_PORT': '9000', 'DEVICE_PORT': '9000'}):
                 with patch("time.sleep"):
                     with self.assertRaises(IOError):
                         server.start()
-                adb.forward.assert_called_once_with(9010, 9000)
+                adb.forward.assert_called_once_with(9000, 9000)
 
     def test_auto_start(self):
-        serial = "76HDGKDN783HD6D"
-        with patch("uiautomator.adb") as adb:
-            with patch("uiautomator.server_port") as server_port:
-                with patch("uiautomator.JsonRPCClient") as JsonRPCClient:
-                    server = AutomatorServer()
-                    server.device_serial = MagicMock()
-                    server.device_serial.return_value = serial
-                    adb.forward_list = {serial: [9008, 9000]}
-                    server_port.return_value = 9000
-                    results = [None, "pong"]
-
-                    def side_effect(*args):
-                        result = results.pop(0)
-                        return result
-                    server.ping = Mock(side_effect=side_effect)
-                    server.start = MagicMock()
-                    server.jsonrpc
-                    JsonRPCClient.assert_called_once_with(server.rpc_uri)
-                    server.start.assert_called_once_with()
-                    server.jsonrpc  # second call will retrieve the stored obj
-                    JsonRPCClient.assert_called_once_with(server.rpc_uri)
-                    server.start.assert_called_once_with()
+        with patch.dict('os.environ', {'LOCAL_PORT': '9000', 'DEVICE_PORT': '9000'}):
+            with patch("uiautomator.JsonRPCClient") as JsonRPCClient:
+                JsonRPCClient.ping.return_value = None
+                server = AutomatorServer()
+                server.start = MagicMock()
+                server.jsonrpc
+                JsonRPCClient.assert_called_once_with(server.rpc_uri)
+                server.start.assert_called_once_with()
+                server.jsonrpc  # second call will retrieve the stored obj
+                JsonRPCClient.assert_called_once_with(server.rpc_uri)
 
     def test_start_ping(self):
-        serial = "76HDGKDN783HD6D"
-        with patch("uiautomator.adb") as adb:
+        with patch.dict('os.environ', {'LOCAL_PORT': '9000', 'DEVICE_PORT': '9000'}):
             with patch("uiautomator.JsonRPCClient") as JsonRPCClient:
-                server = AutomatorServer()
-                server.device_serial = MagicMock()
-                server.device_serial.return_value = serial
-                adb.forward_list = {serial: [9008, 9008]}
                 JsonRPCClient.return_value.ping.return_value = "pong"
+                server = AutomatorServer()
                 self.assertEqual(server.ping(), "pong")
 
     def test_start_ping_none(self):
-        serial = "76HDGKDN783HD6D"
-        with patch("uiautomator.adb") as adb:
-            server = AutomatorServer()
-            server.device_serial = MagicMock()
-            server.device_serial.return_value = serial
-            adb.forward_list = {}
-            self.assertEqual(server.ping(), None)
+        with patch.dict('os.environ', {'LOCAL_PORT': '9000', 'DEVICE_PORT': '9000'}):
+            with patch("uiautomator.JsonRPCClient") as JsonRPCClient:
+                JsonRPCClient.return_value.ping.side_effect = Exception("error")
+                server = AutomatorServer()
+                self.assertEqual(server.ping(), None)
 
 
 class TestAutomatorServer_Stop(unittest.TestCase):
@@ -177,8 +116,8 @@ class TestAutomatorServer_Stop(unittest.TestCase):
     @patch("uiautomator.adb")
     def test_download_and_push_download(self, adb):
         jars = ["bundle.jar", "uiautomator-stub.jar"]
-        with patch("os.path.exists") as exists, \
-             patch("os.mkdir") as mkdir, \
+        with patch("os.path.exists") as exists,\
+             patch("os.mkdir") as mkdir,\
              patch("%s.open" % open.__class__.__module__, mock_open(), create=True) as m_open:
             server = AutomatorServer()
             exists.return_value = False
@@ -189,16 +128,13 @@ class TestAutomatorServer_Stop(unittest.TestCase):
     def test_stop_started_server(self, adb):
         serial = "76HDGKDN783HD6D"
         server = AutomatorServer()
-        server.device_serial = MagicMock()
-        server.device_serial.return_value = serial
-        adb.forward_list = {serial: [9008, 9008]}
-        adb.forward.return_value = 0
         server.uiautomator_process = process = MagicMock()
         process.poll.return_value = None
         server.stop()
         process.wait.assert_called_once_with()
 
-        server.uiautomator_process = process
+        server.uiautomator_process = process = MagicMock()
+        process.poll.return_value = None
         self.urlopen.side_effect = IOError("error")
         server.stop()
         process.kill.assert_called_once_with()
